/*
Copyright (C) 2003-2004 Douglas Thain and the University of Wisconsin
Copyright (C) 2005- The University of Notre Dame
This software is distributed under a BSD-style license.
See the file COPYING for details.
*/

#ifndef CHIRP_GROUP_H
#define CHIRP_GROUP_H

#include <string.h>
#include <stdio.h>
#include <dirent.h>
#include <stdlib.h>
#include <ctype.h>

#include "stringtools.h"
#include "chirp_protocol.h"
#include "int_sizes.h"

#define CACHE_DIR     ".__cache"
#define GROUPNAME_MAX 256
#define USERNAME_MAX  256
#define HOSTNAME_MAX  256
#define NO_CACHING    0
#define DECISION      1
#define GROUP_FILE    2

static INT64_T local_timeout = 100;
struct list *cache_file_index;
struct list *dec_cache_list;
struct list *cache_policy_list;

/* used to store metadata about cached group files */

struct cache_file {
  char grp_name[GROUPNAME_MAX];
  char file_name[USERNAME_MAX];
  time_t expiration;
};

/* used to store metadata about cached group lookup decisions */

struct decision {
  char grp_name[GROUPNAME_MAX];
  char usr_name[USERNAME_MAX];
  time_t expiration;
  INT64_T result;
};

/* used to store metadata about cached policy for a given host */

struct policy {
  char grp_name[GROUPNAME_MAX];
  time_t file_exp;
  time_t dec_exp;
  int file_duration;
  int dec_duration;
  int policy;
};

INT64_T chirp_group_create(char* path);

INT64_T chirp_group_add_user(char* path, char* usr_name);

INT64_T chirp_group_delete_user(char* path, char* usr_name);

INT64_T chirp_group_empty_file_cache(const char* path);

INT64_T chirp_group_check_cache_time(const char* virtual_path, const time_t cache_mod_time);

INT64_T chirp_group_get_policy_from_db(const char* grp_name, unsigned long int* cache_file_exp, unsigned long int* cache_dec_exp);

INT64_T chirp_group_do_lookup(char* grp_name, const char* usr_name);

INT64_T chirp_group_remote_lookup(char* grp_name, const char* usr_name);

INT64_T chirp_group_set_policy(char* path, unsigned long file_cache_exp, unsigned long dec_cache_exp);

void    chirp_group_set_dir(const char* path);

void    chirp_group_set_local_timeout(INT64_T timeout);

INT64_T is_group(char* line);

#endif

